@@ -1,5 +1,3 @@
-// Copyright (c) 2017-present PyO3 Project and Contributors
-
 //! Defines conversions between Rust and Python types.
 use crate::err::{self, PyDowncastError, PyResult};
 #[cfg(feature = "experimental-inspect")]
@@ -22,7 +20,6 @@ use std::ptr::NonNull;
 ///
 /// ```rust
 /// use pyo3::prelude::*;
-/// use pyo3::AsPyPointer;
 /// use pyo3::types::PyString;
 /// use pyo3::ffi;
 ///
@@ -37,12 +34,11 @@ use std::ptr::NonNull;
 ///
 /// # Safety
 ///
-/// It is your responsibility to make sure that the underlying Python object is not dropped too
+/// For callers, it is your responsibility to make sure that the underlying Python object is not dropped too
 /// early. For example, the following code will cause undefined behavior:
 ///
 /// ```rust,no_run
 /// # use pyo3::prelude::*;
-/// # use pyo3::AsPyPointer;
 /// # use pyo3::ffi;
 /// #
 /// Python::with_gil(|py| {
@@ -60,45 +56,15 @@ use std::ptr::NonNull;
 /// and the Python object is dropped immediately after the `0xabad1dea_u32.into_py(py).as_ptr()`
 /// expression is evaluated. To fix the problem, bind Python object to a local variable like earlier
 /// to keep the Python object alive until the end of its scope.
-pub trait AsPyPointer {
+///
+/// Implementors must ensure this returns a valid pointer to a Python object, which borrows a reference count from `&self`.
+pub unsafe trait AsPyPointer {
     /// Returns the underlying FFI pointer as a borrowed pointer.
     fn as_ptr(&self) -> *mut ffi::PyObject;
 }
 
-/// Returns an owned pointer to a Python object.
-///
-/// The returned pointer will be valid until you decrease its reference count. It may be null
-/// depending on the implementation.
-/// It is your responsibility to decrease the reference count of the pointer to avoid leaking memory.
-///
-/// # Examples
-///
-/// ```rust
-/// use pyo3::prelude::*;
-/// use pyo3::types::PyString;
-/// use pyo3::ffi;
-///
-/// Python::with_gil(|py| {
-///     let s: Py<PyString> = "foo".into_py(py);
-///     let ptr = s.into_ptr();
-///
-///     let is_really_a_pystring = unsafe { ffi::PyUnicode_CheckExact(ptr) };
-///     assert_eq!(is_really_a_pystring, 1);
-///
-///     // Because we took ownership of the pointer,
-///     // we must manually decrement it to avoid leaking memory
-///     unsafe { ffi::Py_DECREF(ptr) };
-/// });
-/// ```
-pub trait IntoPyPointer {
-    /// Returns the underlying FFI pointer as an owned pointer.
-    ///
-    /// If `self` has ownership of the underlying pointer, it will "steal" ownership of it.
-    fn into_ptr(self) -> *mut ffi::PyObject;
-}
-
 /// Convert `None` into a null pointer.
-impl<T> AsPyPointer for Option<T>
+unsafe impl<T> AsPyPointer for Option<T>
 where
     T: AsPyPointer,
 {
@@ -109,26 +75,6 @@ where
     }
 }
 
-/// Convert `None` into a null pointer.
-impl<T> IntoPyPointer for Option<T>
-where
-    T: IntoPyPointer,
-{
-    #[inline]
-    fn into_ptr(self) -> *mut ffi::PyObject {
-        self.map_or_else(std::ptr::null_mut, |t| t.into_ptr())
-    }
-}
-
-impl<'a, T> IntoPyPointer for &'a T
-where
-    T: AsPyPointer,
-{
-    fn into_ptr(self) -> *mut ffi::PyObject {
-        unsafe { ffi::_Py_XNewRef(self.as_ptr()) }
-    }
-}
-
 /// Conversion trait that allows various objects to be converted into `PyObject`.
 pub trait ToPyObject {
     /// Converts self into a Python object.
@@ -137,9 +83,8 @@ pub trait ToPyObject {
 
 /// Defines a conversion from a Rust type to a Python object.
 ///
-/// It functions similarly to std's [`Into`](std::convert::Into) trait,
-/// but requires a [GIL token](Python) as an argument.
-/// Many functions and traits internal to PyO3 require this trait as a bound,
+/// It functions similarly to std's [`Into`] trait, but requires a [GIL token](Python)
+/// as an argument. Many functions and traits internal to PyO3 require this trait as a bound,
 /// so a lack of this trait can manifest itself in different error messages.
 ///
 /// # Examples
@@ -214,7 +159,7 @@ pub trait ToPyObject {
 /// # }
 /// ```
 /// Python code will see this as any of the `int`, `string` or `None` objects.
-#[cfg_attr(docsrs, doc(alias = "IntoPyCallbackOutput"))]
+#[doc(alias = "IntoPyCallbackOutput")]
 pub trait IntoPy<T>: Sized {
     /// Performs the conversion.
     fn into_py(self, py: Python<'_>) -> T;
@@ -319,26 +264,20 @@ where
     }
 }
 
-/// `()` is converted to Python `None`.
-impl ToPyObject for () {
-    fn to_object(&self, py: Python<'_>) -> PyObject {
-        py.None()
-    }
-}
-
-impl IntoPy<PyObject> for () {
+impl IntoPy<PyObject> for &'_ PyAny {
+    #[inline]
     fn into_py(self, py: Python<'_>) -> PyObject {
-        py.None()
+        unsafe { PyObject::from_borrowed_ptr(py, self.as_ptr()) }
     }
 }
 
 impl<T> IntoPy<PyObject> for &'_ T
 where
-    T: AsPyPointer,
+    T: AsRef<PyAny>,
 {
     #[inline]
     fn into_py(self, py: Python<'_>) -> PyObject {
-        unsafe { PyObject::from_borrowed_ptr(py, self.as_ptr()) }
+        unsafe { PyObject::from_borrowed_ptr(py, self.as_ref().as_ptr()) }
     }
 }
 
@@ -634,7 +573,7 @@ mod test_no_clone {}
 #[cfg(test)]
 mod tests {
     use crate::types::{IntoPyDict, PyAny, PyDict, PyList};
-    use crate::{AsPyPointer, PyObject, Python, ToPyObject};
+    use crate::{PyObject, Python, ToPyObject};
 
     use super::PyTryFrom;
 
@@ -669,7 +608,7 @@ mod tests {
     #[test]
     fn test_try_from_unchecked() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[1, 2, 3]);
+            let list = PyList::new(py, [1, 2, 3]);
             let val = unsafe { <PyList as PyTryFrom>::try_from_unchecked(list.as_ref()) };
             assert!(list.is(val));
         });
@@ -678,6 +617,7 @@ mod tests {
     #[test]
     fn test_option_as_ptr() {
         Python::with_gil(|py| {
+            use crate::AsPyPointer;
             let mut option: Option<PyObject> = None;
             assert_eq!(option.as_ptr(), std::ptr::null_mut());
 
