@@ -2,6 +2,7 @@
 
 pub mod anyhow;
 pub mod chrono;
+pub mod either;
 pub mod eyre;
 pub mod hashbrown;
 pub mod indexmap;
@@ -9,4 +10,5 @@ pub mod num_bigint;
 pub mod num_complex;
 pub mod rust_decimal;
 pub mod serde;
+pub mod smallvec;
 mod std;
