@@ -1,5 +1,3 @@
-// Copyright (c) 2017-present PyO3 Project and Contributors
-
 //! Exception and warning types defined by Python.
 //!
 //! The structs in this module represent Python's built-in exceptions and
@@ -44,7 +42,6 @@ macro_rules! impl_exception_boilerplate {
         impl ::std::error::Error for $name {
             fn source(&self) -> ::std::option::Option<&(dyn ::std::error::Error + 'static)> {
                 unsafe {
-                    use $crate::AsPyPointer;
                     let cause: &$crate::exceptions::PyBaseException = self
                         .py()
                         .from_owned_ptr_or_opt($crate::ffi::PyException_GetCause(self.as_ptr()))?;
@@ -96,14 +93,13 @@ macro_rules! import_exception {
 
         $crate::pyobject_native_type_core!(
             $name,
-            *$name::type_object_raw($crate::Python::assume_gil_acquired()),
+            $name::type_object_raw,
             #module=::std::option::Option::Some(stringify!($module))
         );
 
         impl $name {
             fn type_object_raw(py: $crate::Python<'_>) -> *mut $crate::ffi::PyTypeObject {
                 use $crate::sync::GILOnceCell;
-                use $crate::AsPyPointer;
                 static TYPE_OBJECT: GILOnceCell<$crate::Py<$crate::types::PyType>> =
                     GILOnceCell::new();
 
@@ -235,14 +231,13 @@ macro_rules! create_exception_type_object {
     ($module: expr, $name: ident, $base: ty, $doc: expr) => {
         $crate::pyobject_native_type_core!(
             $name,
-            *$name::type_object_raw($crate::Python::assume_gil_acquired()),
+            $name::type_object_raw,
             #module=::std::option::Option::Some(stringify!($module))
         );
 
         impl $name {
             fn type_object_raw(py: $crate::Python<'_>) -> *mut $crate::ffi::PyTypeObject {
                 use $crate::sync::GILOnceCell;
-                use $crate::AsPyPointer;
                 static TYPE_OBJECT: GILOnceCell<$crate::Py<$crate::types::PyType>> =
                     GILOnceCell::new();
 
@@ -262,13 +257,13 @@ macro_rules! create_exception_type_object {
 }
 
 macro_rules! impl_native_exception (
-    ($name:ident, $exc_name:ident, $doc:expr, $layout:path) => (
+    ($name:ident, $exc_name:ident, $doc:expr, $layout:path $(, #checkfunction=$checkfunction:path)?) => (
         #[doc = $doc]
         #[allow(clippy::upper_case_acronyms)]
         pub struct $name($crate::PyAny);
 
         $crate::impl_exception_boilerplate!($name);
-        $crate::pyobject_native_type!($name, $layout, *($crate::ffi::$exc_name as *mut $crate::ffi::PyTypeObject));
+        $crate::pyobject_native_type!($name, $layout, |_py| unsafe { $crate::ffi::$exc_name as *mut $crate::ffi::PyTypeObject } $(, #checkfunction=$checkfunction)?);
     );
     ($name:ident, $exc_name:ident, $doc:expr) => (
         impl_native_exception!($name, $exc_name, $doc, $crate::ffi::PyBaseExceptionObject);
@@ -284,7 +279,7 @@ macro_rules! impl_windows_native_exception (
         pub struct $name($crate::PyAny);
 
         $crate::impl_exception_boilerplate!($name);
-        $crate::pyobject_native_type!($name, $layout, *($crate::ffi::$exc_name as *mut $crate::ffi::PyTypeObject));
+        $crate::pyobject_native_type!($name, $layout, |_py| unsafe { $crate::ffi::$exc_name as *mut $crate::ffi::PyTypeObject });
     );
     ($name:ident, $exc_name:ident, $doc:expr) => (
         impl_windows_native_exception!($name, $exc_name, $doc, $crate::ffi::PyBaseExceptionObject);
@@ -361,7 +356,9 @@ Python::with_gil(|py| {
 impl_native_exception!(
     PyBaseException,
     PyExc_BaseException,
-    native_doc!("BaseException")
+    native_doc!("BaseException"),
+    ffi::PyBaseExceptionObject,
+    #checkfunction=ffi::PyExceptionInstance_Check
 );
 impl_native_exception!(PyException, PyExc_Exception, native_doc!("Exception"));
 impl_native_exception!(
@@ -638,6 +635,7 @@ impl PyUnicodeDecodeError {
     /// # Examples
     ///
     /// ```
+    /// #![cfg_attr(invalid_from_utf8_lint, allow(invalid_from_utf8))]
     /// use pyo3::prelude::*;
     /// use pyo3::exceptions::PyUnicodeDecodeError;
     ///
@@ -726,7 +724,7 @@ impl_native_exception!(
 
 #[cfg(test)]
 macro_rules! test_exception {
-    ($exc_ty:ident $(, $constructor:expr)?) => {
+    ($exc_ty:ident $(, |$py:tt| $constructor:expr )?) => {
         #[allow(non_snake_case)]
         #[test]
         fn $exc_ty () {
@@ -737,7 +735,7 @@ macro_rules! test_exception {
                 let err: $crate::PyErr = {
                     None
                     $(
-                        .or(Some($constructor(py)))
+                        .or(Some({ let $py = py; $constructor }))
                     )?
                         .unwrap_or($exc_ty::new_err("a test exception"))
                 };
@@ -772,12 +770,12 @@ pub mod asyncio {
         test_exception!(CancelledError);
         test_exception!(InvalidStateError);
         test_exception!(TimeoutError);
-        test_exception!(IncompleteReadError, |_| {
-            IncompleteReadError::new_err(("partial", "expected"))
-        });
-        test_exception!(LimitOverrunError, |_| {
-            LimitOverrunError::new_err(("message", "consumed"))
-        });
+        test_exception!(IncompleteReadError, |_| IncompleteReadError::new_err((
+            "partial", "expected"
+        )));
+        test_exception!(LimitOverrunError, |_| LimitOverrunError::new_err((
+            "message", "consumed"
+        )));
         test_exception!(QueueEmpty);
         test_exception!(QueueFull);
     }
@@ -813,20 +811,20 @@ mod tests {
             let err: PyErr = gaierror::new_err(());
             let socket = py
                 .import("socket")
-                .map_err(|e| e.print(py))
+                .map_err(|e| e.display(py))
                 .expect("could not import socket");
 
             let d = PyDict::new(py);
             d.set_item("socket", socket)
-                .map_err(|e| e.print(py))
+                .map_err(|e| e.display(py))
                 .expect("could not setitem");
 
             d.set_item("exc", err)
-                .map_err(|e| e.print(py))
+                .map_err(|e| e.display(py))
                 .expect("could not setitem");
 
             py.run("assert isinstance(exc, socket.gaierror)", None, Some(d))
-                .map_err(|e| e.print(py))
+                .map_err(|e| e.display(py))
                 .expect("assertion failed");
         });
     }
@@ -837,15 +835,15 @@ mod tests {
             let err: PyErr = MessageError::new_err(());
             let email = py
                 .import("email")
-                .map_err(|e| e.print(py))
+                .map_err(|e| e.display(py))
                 .expect("could not import email");
 
             let d = PyDict::new(py);
             d.set_item("email", email)
-                .map_err(|e| e.print(py))
+                .map_err(|e| e.display(py))
                 .expect("could not setitem");
             d.set_item("exc", err)
-                .map_err(|e| e.print(py))
+                .map_err(|e| e.display(py))
                 .expect("could not setitem");
 
             py.run(
@@ -853,7 +851,7 @@ mod tests {
                 None,
                 Some(d),
             )
-            .map_err(|e| e.print(py))
+            .map_err(|e| e.display(py))
             .expect("assertion failed");
         });
     }
@@ -1016,6 +1014,7 @@ mod tests {
     #[test]
     fn unicode_decode_error() {
         let invalid_utf8 = b"fo\xd8o";
+        #[cfg_attr(invalid_from_utf8_lint, allow(invalid_from_utf8))]
         let err = std::str::from_utf8(invalid_utf8).expect_err("should be invalid utf8");
         Python::with_gil(|py| {
             let decode_err = PyUnicodeDecodeError::new_utf8(py, invalid_utf8, err).unwrap();
@@ -1035,9 +1034,10 @@ mod tests {
         });
     }
     #[cfg(Py_3_11)]
-    test_exception!(PyBaseExceptionGroup, |_| {
-        PyBaseExceptionGroup::new_err(("msg", vec![PyValueError::new_err("err")]))
-    });
+    test_exception!(PyBaseExceptionGroup, |_| PyBaseExceptionGroup::new_err((
+        "msg",
+        vec![PyValueError::new_err("err")]
+    )));
     test_exception!(PyBaseException);
     test_exception!(PyException);
     test_exception!(PyStopAsyncIteration);
@@ -1071,13 +1071,13 @@ mod tests {
     test_exception!(PyUnicodeError);
     test_exception!(PyUnicodeDecodeError, |py| {
         let invalid_utf8 = b"fo\xd8o";
+        #[cfg_attr(invalid_from_utf8_lint, allow(invalid_from_utf8))]
         let err = std::str::from_utf8(invalid_utf8).expect_err("should be invalid utf8");
         PyErr::from_value(PyUnicodeDecodeError::new_utf8(py, invalid_utf8, err).unwrap())
     });
-    test_exception!(PyUnicodeEncodeError, |py: Python<'_>| {
-        py.eval("chr(40960).encode('ascii')", None, None)
-            .unwrap_err()
-    });
+    test_exception!(PyUnicodeEncodeError, |py| py
+        .eval("chr(40960).encode('ascii')", None, None)
+        .unwrap_err());
     test_exception!(PyUnicodeTranslateError, |_| {
         PyUnicodeTranslateError::new_err(("\u{3042}", 0, 1, "ouch"))
     });
