@@ -1,3 +1,5 @@
+#![allow(missing_docs)]
+
 //! Internals of PyO3 which are accessed by code expanded from PyO3's procedural macros.
 //!
 //! Usage of any of these APIs in downstream code is implicitly acknowledging that these
@@ -8,7 +10,6 @@ pub mod deprecations;
 pub mod extract_argument;
 pub mod freelist;
 pub mod frompyobject;
-pub mod ghost;
 pub(crate) mod not_send;
 pub mod panic;
 pub mod pycell;
@@ -16,3 +17,6 @@ pub mod pyclass;
 pub mod pyfunction;
 pub mod pymethods;
 pub mod pymodule;
+#[doc(hidden)]
+pub mod trampoline;
+pub mod wrap;
