@@ -1,7 +1,3 @@
-// Copyright (c) 2017-present PyO3 Project and Contributors
-//
-// based on Daniel Grunwald's https://github.com/dgrunwald/rust-cpython
-
 //! Fundamental properties of objects tied to the Python interpreter.
 //!
 //! The Python interpreter is not threadsafe. To protect the Python interpreter in multithreaded
@@ -122,12 +118,11 @@
 use crate::err::{self, PyDowncastError, PyErr, PyResult};
 use crate::gil::{GILGuard, GILPool, SuspendGIL};
 use crate::impl_::not_send::NotSend;
-use crate::types::{PyAny, PyDict, PyModule, PyString, PyType};
-use crate::version::PythonVersionInfo;
-use crate::{
-    ffi, AsPyPointer, FromPyPointer, IntoPy, IntoPyPointer, Py, PyNativeType, PyObject, PyTryFrom,
-    PyTypeInfo,
+use crate::types::{
+    PyAny, PyDict, PyEllipsis, PyModule, PyNone, PyNotImplemented, PyString, PyType,
 };
+use crate::version::PythonVersionInfo;
+use crate::{ffi, FromPyPointer, IntoPy, Py, PyNativeType, PyObject, PyTryFrom, PyTypeInfo};
 use std::ffi::{CStr, CString};
 use std::marker::PhantomData;
 use std::os::raw::c_int;
@@ -190,84 +185,87 @@ pub unsafe trait Ungil {}
 #[cfg(not(feature = "nightly"))]
 unsafe impl<T: Send> Ungil for T {}
 
-/// Types that are safe to access while the GIL is not held.
-///
-/// # Safety
-///
-/// The type must not carry borrowed Python references or, if it does, not allow access to them if
-/// the GIL is not held.
-///
-/// See the [module-level documentation](self) for more information.
-///
-/// # Examples
-///
-/// Types which are `Ungil` cannot be used in contexts where the GIL was released, e.g.
-///
-/// ```compile_fail
-/// # use pyo3::prelude::*;
-/// # use pyo3::types::PyString;
-/// Python::with_gil(|py| {
-///     let string = PyString::new(py, "foo");
-///
-///     py.allow_threads(|| {
-///         println!("{:?}", string);
-///     });
-/// });
-/// ```
-///
-/// This applies to the GIL token `Python` itself as well, e.g.
-///
-/// ```compile_fail
-/// # use pyo3::prelude::*;
-/// Python::with_gil(|py| {
-///     py.allow_threads(|| {
-///         drop(py);
-///     });
-/// });
-/// ```
-///
-/// On nightly Rust, this is not based on the [`Send`] auto trait and hence we are able
-/// to prevent incorrectly circumventing it using e.g. the [`send_wrapper`](https://docs.rs/send_wrapper/) crate:
-///
-/// ```compile_fail
-/// # use pyo3::prelude::*;
-/// # use pyo3::types::PyString;
-/// use send_wrapper::SendWrapper;
-///
-/// Python::with_gil(|py| {
-///     let string = PyString::new(py, "foo");
-///
-///     let wrapped = SendWrapper::new(string);
-///
-///     py.allow_threads(|| {
-///         let sneaky: &PyString = *wrapped;
-///
-///         println!("{:?}", sneaky);
-///     });
-/// });
-/// ```
-///
-/// This also enables using non-[`Send`] types in `allow_threads`,
-/// at least if they are not also bound to the GIL:
-///
-/// ```rust
-/// # use pyo3::prelude::*;
-/// use std::rc::Rc;
-///
-/// Python::with_gil(|py| {
-///     let rc = Rc::new(42);
-///
-///     py.allow_threads(|| {
-///         println!("{:?}", rc);
-///     });
-/// });
-/// ```
 #[cfg(feature = "nightly")]
-pub unsafe auto trait Ungil {}
+mod nightly {
+    macro_rules! define {
+        ($($tt:tt)*) => { $($tt)* }
+    }
 
-#[cfg(feature = "nightly")]
-mod negative_impls {
-    use super::Ungil;
+    define! {
+        /// Types that are safe to access while the GIL is not held.
+        ///
+        /// # Safety
+        ///
+        /// The type must not carry borrowed Python references or, if it does, not allow access to them if
+        /// the GIL is not held.
+        ///
+        /// See the [module-level documentation](self) for more information.
+        ///
+        /// # Examples
+        ///
+        /// Types which are `Ungil` cannot be used in contexts where the GIL was released, e.g.
+        ///
+        /// ```compile_fail
+        /// # use pyo3::prelude::*;
+        /// # use pyo3::types::PyString;
+        /// Python::with_gil(|py| {
+        ///     let string = PyString::new(py, "foo");
+        ///
+        ///     py.allow_threads(|| {
+        ///         println!("{:?}", string);
+        ///     });
+        /// });
+        /// ```
+        ///
+        /// This applies to the GIL token `Python` itself as well, e.g.
+        ///
+        /// ```compile_fail
+        /// # use pyo3::prelude::*;
+        /// Python::with_gil(|py| {
+        ///     py.allow_threads(|| {
+        ///         drop(py);
+        ///     });
+        /// });
+        /// ```
+        ///
+        /// On nightly Rust, this is not based on the [`Send`] auto trait and hence we are able
+        /// to prevent incorrectly circumventing it using e.g. the [`send_wrapper`](https://docs.rs/send_wrapper/) crate:
+        ///
+        /// ```compile_fail
+        /// # use pyo3::prelude::*;
+        /// # use pyo3::types::PyString;
+        /// use send_wrapper::SendWrapper;
+        ///
+        /// Python::with_gil(|py| {
+        ///     let string = PyString::new(py, "foo");
+        ///
+        ///     let wrapped = SendWrapper::new(string);
+        ///
+        ///     py.allow_threads(|| {
+        ///         let sneaky: &PyString = *wrapped;
+        ///
+        ///         println!("{:?}", sneaky);
+        ///     });
+        /// });
+        /// ```
+        ///
+        /// This also enables using non-[`Send`] types in `allow_threads`,
+        /// at least if they are not also bound to the GIL:
+        ///
+        /// ```rust
+        /// # use pyo3::prelude::*;
+        /// use std::rc::Rc;
+        ///
+        /// Python::with_gil(|py| {
+        ///     let rc = Rc::new(42);
+        ///
+        ///     py.allow_threads(|| {
+        ///         println!("{:?}", rc);
+        ///     });
+        /// });
+        /// ```
+        pub unsafe auto trait Ungil {}
+    }
 
     impl !Ungil for crate::Python<'_> {}
 
@@ -294,6 +292,9 @@ mod negative_impls {
     impl !Ungil for crate::ffi::PyArena {}
 }
 
+#[cfg(feature = "nightly")]
+pub use nightly::Ungil;
+
 /// A marker token that represents holding the GIL.
 ///
 /// It serves three main purposes:
@@ -548,6 +549,9 @@ impl<'py> Python<'py> {
     /// If `globals` is `None`, it defaults to Python module `__main__`.
     /// If `locals` is `None`, it defaults to the value of `globals`.
     ///
+    /// If `globals` doesn't contain `__builtins__`, default `__builtins__`
+    /// will be added automatically.
+    ///
     /// # Examples
     ///
     /// ```
@@ -572,6 +576,9 @@ impl<'py> Python<'py> {
     /// If `globals` is `None`, it defaults to Python module `__main__`.
     /// If `locals` is `None`, it defaults to the value of `globals`.
     ///
+    /// If `globals` doesn't contain `__builtins__`, default `__builtins__`
+    /// will be added automatically.
+    ///
     /// # Examples
     /// ```
     /// use pyo3::{
@@ -590,7 +597,7 @@ impl<'py> Python<'py> {
     ///         Some(locals),
     ///     )
     ///     .unwrap();
-    ///     let ret = locals.get_item("ret").unwrap();
+    ///     let ret = locals.get_item("ret").unwrap().unwrap();
     ///     let b64: &PyBytes = ret.downcast().unwrap();
     ///     assert_eq!(b64.as_bytes(), b"SGVsbG8gUnVzdCE=");
     /// });
@@ -632,9 +639,32 @@ impl<'py> Python<'py> {
             }
 
             let globals = globals
-                .map(AsPyPointer::as_ptr)
+                .map(|dict| dict.as_ptr())
                 .unwrap_or_else(|| ffi::PyModule_GetDict(mptr));
-            let locals = locals.map(AsPyPointer::as_ptr).unwrap_or(globals);
+            let locals = locals.map(|dict| dict.as_ptr()).unwrap_or(globals);
+
+            // If `globals` don't provide `__builtins__`, most of the code will fail if Python
+            // version is <3.10. That's probably not what user intended, so insert `__builtins__`
+            // for them.
+            //
+            // See also:
+            // - https://github.com/python/cpython/pull/24564 (the same fix in CPython 3.10)
+            // - https://github.com/PyO3/pyo3/issues/3370
+            let builtins_s = crate::intern!(self, "__builtins__").as_ptr();
+            let has_builtins = ffi::PyDict_Contains(globals, builtins_s);
+            if has_builtins == -1 {
+                return Err(PyErr::fetch(self));
+            }
+            if has_builtins == 0 {
+                // Inherit current builtins.
+                let builtins = ffi::PyEval_GetBuiltins();
+
+                // `PyDict_SetItem` doesn't take ownership of `builtins`, but `PyEval_GetBuiltins`
+                // seems to return a borrowed reference, so no leak here.
+                if ffi::PyDict_SetItem(globals, builtins_s, builtins) == -1 {
+                    return Err(PyErr::fetch(self));
+                }
+            }
 
             let code_obj = ffi::Py_CompileString(code.as_ptr(), "<string>\0".as_ptr() as _, start);
             if code_obj.is_null() {
@@ -668,21 +698,21 @@ impl<'py> Python<'py> {
     #[allow(non_snake_case)] // the Python keyword starts with uppercase
     #[inline]
     pub fn None(self) -> PyObject {
-        unsafe { PyObject::from_borrowed_ptr(self, ffi::Py_None()) }
+        PyNone::get(self).into()
     }
 
     /// Gets the Python builtin value `Ellipsis`, or `...`.
     #[allow(non_snake_case)] // the Python keyword starts with uppercase
     #[inline]
     pub fn Ellipsis(self) -> PyObject {
-        unsafe { PyObject::from_borrowed_ptr(self, ffi::Py_Ellipsis()) }
+        PyEllipsis::get(self).into()
     }
 
     /// Gets the Python builtin value `NotImplemented`.
     #[allow(non_snake_case)] // the Python keyword starts with uppercase
     #[inline]
     pub fn NotImplemented(self) -> PyObject {
-        unsafe { PyObject::from_borrowed_ptr(self, ffi::Py_NotImplemented()) }
+        PyNotImplemented::get(self).into()
     }
 
     /// Gets the running Python interpreter version as a string.
@@ -877,11 +907,13 @@ impl<'py> Python<'py> {
     /// As Python's [`signal`][2] API allows users to define custom signal handlers, calling this
     /// function allows arbitrary Python code inside signal handlers to run.
     ///
+    /// If the function is called from a non-main thread, or under a non-main Python interpreter,
+    /// it does nothing yet still returns `Ok(())`.
+    ///
     /// [1]: https://docs.python.org/3/c-api/exceptions.html?highlight=pyerr_checksignals#c.PyErr_CheckSignals
     /// [2]: https://docs.python.org/3/library/signal.html
     pub fn check_signals(self) -> PyResult<()> {
-        let v = unsafe { ffi::PyErr_CheckSignals() };
-        err::error_on_minusone(self, v)
+        err::error_on_minusone(self, unsafe { ffi::PyErr_CheckSignals() })
     }
 
     /// Create a new pool for managing PyO3's owned references.
@@ -944,6 +976,72 @@ impl<'py> Python<'py> {
     }
 }
 
+impl Python<'_> {
+    /// Creates a scope using a new pool for managing PyO3's owned references.
+    ///
+    /// This is a safe alterantive to [`new_pool`][Self::new_pool] as
+    /// it limits the closure to using the new GIL token at the cost of
+    /// being unable to capture existing GIL-bound references.
+    ///
+    /// Note that on stable Rust, this API suffers from the same the `SendWrapper` loophole
+    /// as [`allow_threads`][Self::allow_threads], c.f. the documentation of the [`Ungil`] trait,
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// # use pyo3::prelude::*;
+    /// Python::with_gil(|py| {
+    ///     // Some long-running process like a webserver, which never releases the GIL.
+    ///     loop {
+    ///         // Create a new scope, so that PyO3 can clear memory at the end of the loop.
+    ///         py.with_pool(|py| {
+    ///             // do stuff...
+    ///         });
+    /// #       break;  // Exit the loop so that doctest terminates!
+    ///     }
+    /// });
+    /// ```
+    ///
+    /// The `Ungil` bound on the closure does prevent hanging on to existing GIL-bound references
+    ///
+    /// ```compile_fail
+    /// # use pyo3::prelude::*;
+    /// # use pyo3::types::PyString;
+    ///
+    /// Python::with_gil(|py| {
+    ///     let old_str = PyString::new(py, "a message from the past");
+    ///
+    ///     py.with_pool(|_py| {
+    ///         print!("{:?}", old_str);
+    ///     });
+    /// });
+    /// ```
+    ///
+    /// or continuing to use the old GIL token
+    ///
+    /// ```compile_fail
+    /// # use pyo3::prelude::*;
+    ///
+    /// Python::with_gil(|old_py| {
+    ///     old_py.with_pool(|_new_py| {
+    ///         let _none = old_py.None();
+    ///     });
+    /// });
+    /// ```
+    #[inline]
+    pub fn with_pool<F, R>(&self, f: F) -> R
+    where
+        F: for<'py> FnOnce(Python<'py>) -> R + Ungil,
+    {
+        // SAFETY: The closure is `Ungil`,
+        // i.e. it does not capture any GIL-bound references
+        // and accesses only the newly created GIL token.
+        let pool = unsafe { GILPool::new() };
+
+        f(pool.python())
+    }
+}
+
 impl<'unbound> Python<'unbound> {
     /// Unsafely creates a Python token with an unbounded lifetime.
     ///
@@ -970,7 +1068,7 @@ impl<'unbound> Python<'unbound> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::types::{IntoPyDict, PyList};
+    use crate::types::{IntoPyDict, PyDict, PyList};
     use crate::Py;
     use std::sync::Arc;
 
@@ -980,7 +1078,7 @@ mod tests {
             // Make sure builtin names are accessible
             let v: i32 = py
                 .eval("min(1, 2)", None, None)
-                .map_err(|e| e.print(py))
+                .map_err(|e| e.display(py))
                 .unwrap()
                 .extract()
                 .unwrap();
@@ -1053,7 +1151,7 @@ mod tests {
 
             // If allow_threads is implemented correctly, this thread still owns the GIL here
             // so the following Python calls should not cause crashes.
-            let list = PyList::new(py, &[1, 2, 3, 4]);
+            let list = PyList::new(py, [1, 2, 3, 4]);
             assert_eq!(list.extract::<Vec<i32>>().unwrap(), vec![1, 2, 3, 4]);
         });
     }
@@ -1097,9 +1195,23 @@ mod tests {
         Python::with_gil(|py| {
             assert_eq!(py.Ellipsis().to_string(), "Ellipsis");
 
-            let v = py.eval("...", None, None).map_err(|e| e.print(py)).unwrap();
+            let v = py
+                .eval("...", None, None)
+                .map_err(|e| e.display(py))
+                .unwrap();
 
             assert!(v.eq(py.Ellipsis()).unwrap());
         });
     }
+
+    #[test]
+    fn test_py_run_inserts_globals() {
+        Python::with_gil(|py| {
+            let namespace = PyDict::new(py);
+            py.run("class Foo: pass", Some(namespace), Some(namespace))
+                .unwrap();
+            assert!(matches!(namespace.get_item("Foo"), Ok(Some(..))));
+            assert!(matches!(namespace.get_item("__builtins__"), Ok(Some(..))));
+        })
+    }
 }
