@@ -1,5 +1,3 @@
-// Copyright (c) 2022-present PyO3 Project and Contributors
-
 use crate::ffi;
 use crate::PyAny;
 
@@ -9,6 +7,6 @@ pub struct PyCode(PyAny);
 
 pyobject_native_type_core!(
     PyCode,
-    ffi::PyCode_Type,
+    pyobject_native_static_type_object!(ffi::PyCode_Type),
     #checkfunction=ffi::PyCode_Check
 );
