@@ -1,20 +1,17 @@
-// Copyright (c) 2017-present PyO3 Project and Contributors
-//
-// based on Daniel Grunwald's https://github.com/dgrunwald/rust-cpython
-
 use std::convert::TryInto;
+use std::iter::FusedIterator;
 
 use crate::err::{self, PyResult};
 use crate::ffi::{self, Py_ssize_t};
 use crate::internal_tricks::get_ssize_index;
 use crate::types::{PySequence, PyTuple};
-use crate::{AsPyPointer, IntoPyPointer, Py, PyAny, PyObject, Python, ToPyObject};
+use crate::{Py, PyAny, PyObject, Python, ToPyObject};
 
 /// Represents a Python `list`.
 #[repr(transparent)]
 pub struct PyList(PyAny);
 
-pyobject_native_type_core!(PyList, ffi::PyList_Type, #checkfunction=ffi::PyList_Check);
+pyobject_native_type_core!(PyList, pyobject_native_static_type_object!(ffi::PyList_Type), #checkfunction=ffi::PyList_Check);
 
 #[inline]
 #[track_caller]
@@ -123,7 +120,7 @@ impl PyList {
     /// ```
     /// use pyo3::{prelude::*, types::PyList};
     /// Python::with_gil(|py| {
-    ///     let list = PyList::new(py, &[2, 3, 5, 7]);
+    ///     let list = PyList::new(py, [2, 3, 5, 7]);
     ///     let obj = list.get_item(0);
     ///     assert_eq!(obj.unwrap().extract::<i32>().unwrap(), 2);
     /// });
@@ -171,16 +168,13 @@ impl PyList {
     where
         I: ToPyObject,
     {
-        unsafe {
-            err::error_on_minusone(
-                self.py(),
-                ffi::PyList_SetItem(
-                    self.as_ptr(),
-                    get_ssize_index(index),
-                    item.to_object(self.py()).into_ptr(),
-                ),
-            )
+        fn inner(list: &PyList, index: usize, item: PyObject) -> PyResult<()> {
+            err::error_on_minusone(list.py(), unsafe {
+                ffi::PyList_SetItem(list.as_ptr(), get_ssize_index(index), item.into_ptr())
+            })
         }
+
+        inner(self, index, item.to_object(self.py()))
     }
 
     /// Deletes the `index`th element of self.
@@ -196,17 +190,14 @@ impl PyList {
     /// This is equivalent to the Python statement `self[low:high] = v`.
     #[inline]
     pub fn set_slice(&self, low: usize, high: usize, seq: &PyAny) -> PyResult<()> {
-        unsafe {
-            err::error_on_minusone(
-                self.py(),
-                ffi::PyList_SetSlice(
-                    self.as_ptr(),
-                    get_ssize_index(low),
-                    get_ssize_index(high),
-                    seq.as_ptr(),
-                ),
+        err::error_on_minusone(self.py(), unsafe {
+            ffi::PyList_SetSlice(
+                self.as_ptr(),
+                get_ssize_index(low),
+                get_ssize_index(high),
+                seq.as_ptr(),
             )
-        }
+        })
     }
 
     /// Deletes the slice from `low` to `high` from `self`.
@@ -222,13 +213,13 @@ impl PyList {
     where
         I: ToPyObject,
     {
-        let py = self.py();
-        unsafe {
-            err::error_on_minusone(
-                py,
-                ffi::PyList_Append(self.as_ptr(), item.to_object(py).as_ptr()),
-            )
+        fn inner(list: &PyList, item: PyObject) -> PyResult<()> {
+            err::error_on_minusone(list.py(), unsafe {
+                ffi::PyList_Append(list.as_ptr(), item.as_ptr())
+            })
         }
+
+        inner(self, item.to_object(self.py()))
     }
 
     /// Inserts an item at the specified index.
@@ -238,17 +229,13 @@ impl PyList {
     where
         I: ToPyObject,
     {
-        let py = self.py();
-        unsafe {
-            err::error_on_minusone(
-                py,
-                ffi::PyList_Insert(
-                    self.as_ptr(),
-                    get_ssize_index(index),
-                    item.to_object(py).as_ptr(),
-                ),
-            )
+        fn inner(list: &PyList, index: usize, item: PyObject) -> PyResult<()> {
+            err::error_on_minusone(list.py(), unsafe {
+                ffi::PyList_Insert(list.as_ptr(), get_ssize_index(index), item.as_ptr())
+            })
         }
+
+        inner(self, index, item.to_object(self.py()))
     }
 
     /// Determines if self contains `value`.
@@ -278,17 +265,18 @@ impl PyList {
         PyListIterator {
             list: self,
             index: 0,
+            length: self.len(),
         }
     }
 
     /// Sorts the list in-place. Equivalent to the Python expression `l.sort()`.
     pub fn sort(&self) -> PyResult<()> {
-        unsafe { err::error_on_minusone(self.py(), ffi::PyList_Sort(self.as_ptr())) }
+        err::error_on_minusone(self.py(), unsafe { ffi::PyList_Sort(self.as_ptr()) })
     }
 
     /// Reverses the list in-place. Equivalent to the Python expression `l.reverse()`.
     pub fn reverse(&self) -> PyResult<()> {
-        unsafe { err::error_on_minusone(self.py(), ffi::PyList_Reverse(self.as_ptr())) }
+        err::error_on_minusone(self.py(), unsafe { ffi::PyList_Reverse(self.as_ptr()) })
     }
 
     /// Return a new tuple containing the contents of the list; equivalent to the Python expression `tuple(list)`.
@@ -305,18 +293,28 @@ index_impls!(PyList, "list", PyList::len, PyList::get_slice);
 pub struct PyListIterator<'a> {
     list: &'a PyList,
     index: usize,
+    length: usize,
+}
+
+impl<'a> PyListIterator<'a> {
+    unsafe fn get_item(&self, index: usize) -> &'a PyAny {
+        #[cfg(any(Py_LIMITED_API, PyPy))]
+        let item = self.list.get_item(index).expect("list.get failed");
+        #[cfg(not(any(Py_LIMITED_API, PyPy)))]
+        let item = self.list.get_item_unchecked(index);
+        item
+    }
 }
 
 impl<'a> Iterator for PyListIterator<'a> {
     type Item = &'a PyAny;
 
     #[inline]
-    fn next(&mut self) -> Option<&'a PyAny> {
-        if self.index < self.list.len() {
-            #[cfg(any(Py_LIMITED_API, PyPy))]
-            let item = self.list.get_item(self.index).expect("list.get failed");
-            #[cfg(not(any(Py_LIMITED_API, PyPy)))]
-            let item = unsafe { self.list.get_item_unchecked(self.index) };
+    fn next(&mut self) -> Option<Self::Item> {
+        let length = self.length.min(self.list.len());
+
+        if self.index < length {
+            let item = unsafe { self.get_item(self.index) };
             self.index += 1;
             Some(item)
         } else {
@@ -331,13 +329,30 @@ impl<'a> Iterator for PyListIterator<'a> {
     }
 }
 
-impl<'a> ExactSizeIterator for PyListIterator<'a> {
-    fn len(&self) -> usize {
-        self.list.len().saturating_sub(self.index)
+impl<'a> DoubleEndedIterator for PyListIterator<'a> {
+    #[inline]
+    fn next_back(&mut self) -> Option<Self::Item> {
+        let length = self.length.min(self.list.len());
+
+        if self.index < length {
+            let item = unsafe { self.get_item(length - 1) };
+            self.length = length - 1;
+            Some(item)
+        } else {
+            None
+        }
     }
 }
 
-impl<'a> std::iter::IntoIterator for &'a PyList {
+impl<'a> ExactSizeIterator for PyListIterator<'a> {
+    fn len(&self) -> usize {
+        self.length.saturating_sub(self.index)
+    }
+}
+
+impl FusedIterator for PyListIterator<'_> {}
+
+impl<'a> IntoIterator for &'a PyList {
     type Item = &'a PyAny;
     type IntoIter = PyListIterator<'a>;
 
@@ -355,7 +370,7 @@ mod tests {
     #[test]
     fn test_new() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5, 7]);
+            let list = PyList::new(py, [2, 3, 5, 7]);
             assert_eq!(2, list[0].extract::<i32>().unwrap());
             assert_eq!(3, list[1].extract::<i32>().unwrap());
             assert_eq!(5, list[2].extract::<i32>().unwrap());
@@ -366,7 +381,7 @@ mod tests {
     #[test]
     fn test_len() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[1, 2, 3, 4]);
+            let list = PyList::new(py, [1, 2, 3, 4]);
             assert_eq!(4, list.len());
         });
     }
@@ -374,7 +389,7 @@ mod tests {
     #[test]
     fn test_get_item() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5, 7]);
+            let list = PyList::new(py, [2, 3, 5, 7]);
             assert_eq!(2, list.get_item(0).unwrap().extract::<i32>().unwrap());
             assert_eq!(3, list.get_item(1).unwrap().extract::<i32>().unwrap());
             assert_eq!(5, list.get_item(2).unwrap().extract::<i32>().unwrap());
@@ -385,7 +400,7 @@ mod tests {
     #[test]
     fn test_get_slice() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5, 7]);
+            let list = PyList::new(py, [2, 3, 5, 7]);
             let slice = list.get_slice(1, 3);
             assert_eq!(2, slice.len());
             let slice = list.get_slice(1, 7);
@@ -396,7 +411,7 @@ mod tests {
     #[test]
     fn test_set_item() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5, 7]);
+            let list = PyList::new(py, [2, 3, 5, 7]);
             let val = 42i32.to_object(py);
             let val2 = 42i32.to_object(py);
             assert_eq!(2, list[0].extract::<i32>().unwrap());
@@ -409,25 +424,25 @@ mod tests {
     #[test]
     fn test_set_item_refcnt() {
         Python::with_gil(|py| {
+            let obj = py.eval("object()", None, None).unwrap();
             let cnt;
             {
                 let _pool = unsafe { crate::GILPool::new() };
                 let v = vec![2];
                 let ob = v.to_object(py);
                 let list: &PyList = ob.downcast(py).unwrap();
-                let none = py.None();
-                cnt = none.get_refcnt(py);
-                list.set_item(0, none).unwrap();
+                cnt = obj.get_refcnt();
+                list.set_item(0, obj).unwrap();
             }
 
-            assert_eq!(cnt, py.None().get_refcnt(py));
+            assert_eq!(cnt, obj.get_refcnt());
         });
     }
 
     #[test]
     fn test_insert() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5, 7]);
+            let list = PyList::new(py, [2, 3, 5, 7]);
             let val = 42i32.to_object(py);
             let val2 = 43i32.to_object(py);
             assert_eq!(4, list.len());
@@ -445,22 +460,22 @@ mod tests {
     fn test_insert_refcnt() {
         Python::with_gil(|py| {
             let cnt;
+            let obj = py.eval("object()", None, None).unwrap();
             {
                 let _pool = unsafe { crate::GILPool::new() };
                 let list = PyList::empty(py);
-                let none = py.None();
-                cnt = none.get_refcnt(py);
-                list.insert(0, none).unwrap();
+                cnt = obj.get_refcnt();
+                list.insert(0, obj).unwrap();
             }
 
-            assert_eq!(cnt, py.None().get_refcnt(py));
+            assert_eq!(cnt, obj.get_refcnt());
         });
     }
 
     #[test]
     fn test_append() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2]);
+            let list = PyList::new(py, [2]);
             list.append(3).unwrap();
             assert_eq!(2, list[0].extract::<i32>().unwrap());
             assert_eq!(3, list[1].extract::<i32>().unwrap());
@@ -471,14 +486,14 @@ mod tests {
     fn test_append_refcnt() {
         Python::with_gil(|py| {
             let cnt;
+            let obj = py.eval("object()", None, None).unwrap();
             {
                 let _pool = unsafe { crate::GILPool::new() };
                 let list = PyList::empty(py);
-                let none = py.None();
-                cnt = none.get_refcnt(py);
-                list.append(none).unwrap();
+                cnt = obj.get_refcnt();
+                list.append(obj).unwrap();
             }
-            assert_eq!(cnt, py.None().get_refcnt(py));
+            assert_eq!(cnt, obj.get_refcnt());
         });
     }
 
@@ -488,7 +503,7 @@ mod tests {
             let v = vec![2, 3, 5, 7];
             let list = PyList::new(py, &v);
             let mut idx = 0;
-            for el in list.iter() {
+            for el in list {
                 assert_eq!(v[idx], el.extract::<i32>().unwrap());
                 idx += 1;
             }
@@ -508,17 +523,45 @@ mod tests {
             iter.next();
             assert_eq!(iter.size_hint(), (v.len() - 1, Some(v.len() - 1)));
 
-            // Exhust iterator.
+            // Exhaust iterator.
             for _ in &mut iter {}
 
             assert_eq!(iter.size_hint(), (0, Some(0)));
         });
     }
 
+    #[test]
+    fn test_iter_rev() {
+        Python::with_gil(|py| {
+            let v = vec![2, 3, 5, 7];
+            let ob = v.to_object(py);
+            let list: &PyList = ob.downcast(py).unwrap();
+
+            let mut iter = list.iter().rev();
+
+            assert_eq!(iter.size_hint(), (4, Some(4)));
+
+            assert_eq!(iter.next().unwrap().extract::<i32>().unwrap(), 7);
+            assert_eq!(iter.size_hint(), (3, Some(3)));
+
+            assert_eq!(iter.next().unwrap().extract::<i32>().unwrap(), 5);
+            assert_eq!(iter.size_hint(), (2, Some(2)));
+
+            assert_eq!(iter.next().unwrap().extract::<i32>().unwrap(), 3);
+            assert_eq!(iter.size_hint(), (1, Some(1)));
+
+            assert_eq!(iter.next().unwrap().extract::<i32>().unwrap(), 2);
+            assert_eq!(iter.size_hint(), (0, Some(0)));
+
+            assert!(iter.next().is_none());
+            assert!(iter.next().is_none());
+        });
+    }
+
     #[test]
     fn test_into_iter() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[1, 2, 3, 4]);
+            let list = PyList::new(py, [1, 2, 3, 4]);
             for (i, item) in list.iter().enumerate() {
                 assert_eq!((i + 1) as i32, item.extract::<i32>().unwrap());
             }
@@ -582,7 +625,7 @@ mod tests {
     #[test]
     fn test_list_get_item_invalid_index() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5, 7]);
+            let list = PyList::new(py, [2, 3, 5, 7]);
             let obj = list.get_item(5);
             assert!(obj.is_err());
             assert_eq!(
@@ -595,7 +638,7 @@ mod tests {
     #[test]
     fn test_list_get_item_sanity() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5, 7]);
+            let list = PyList::new(py, [2, 3, 5, 7]);
             let obj = list.get_item(0);
             assert_eq!(obj.unwrap().extract::<i32>().unwrap(), 2);
         });
@@ -605,7 +648,7 @@ mod tests {
     #[test]
     fn test_list_get_item_unchecked_sanity() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5, 7]);
+            let list = PyList::new(py, [2, 3, 5, 7]);
             let obj = unsafe { list.get_item_unchecked(0) };
             assert_eq!(obj.extract::<i32>().unwrap(), 2);
         });
@@ -614,7 +657,7 @@ mod tests {
     #[test]
     fn test_list_index_trait() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5]);
+            let list = PyList::new(py, [2, 3, 5]);
             assert_eq!(2, list[0].extract::<i32>().unwrap());
             assert_eq!(3, list[1].extract::<i32>().unwrap());
             assert_eq!(5, list[2].extract::<i32>().unwrap());
@@ -625,7 +668,7 @@ mod tests {
     #[should_panic]
     fn test_list_index_trait_panic() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5]);
+            let list = PyList::new(py, [2, 3, 5]);
             let _ = &list[7];
         });
     }
@@ -633,7 +676,7 @@ mod tests {
     #[test]
     fn test_list_index_trait_ranges() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5]);
+            let list = PyList::new(py, [2, 3, 5]);
             assert_eq!(vec![3, 5], list[1..3].extract::<Vec<i32>>().unwrap());
             assert_eq!(Vec::<i32>::new(), list[3..3].extract::<Vec<i32>>().unwrap());
             assert_eq!(vec![3, 5], list[1..].extract::<Vec<i32>>().unwrap());
@@ -649,7 +692,7 @@ mod tests {
     #[should_panic = "range start index 5 out of range for list of length 3"]
     fn test_list_index_trait_range_panic_start() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5]);
+            let list = PyList::new(py, [2, 3, 5]);
             list[5..10].extract::<Vec<i32>>().unwrap();
         })
     }
@@ -658,7 +701,7 @@ mod tests {
     #[should_panic = "range end index 10 out of range for list of length 3"]
     fn test_list_index_trait_range_panic_end() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5]);
+            let list = PyList::new(py, [2, 3, 5]);
             list[1..10].extract::<Vec<i32>>().unwrap();
         })
     }
@@ -667,7 +710,7 @@ mod tests {
     #[should_panic = "slice index starts at 2 but ends at 1"]
     fn test_list_index_trait_range_panic_wrong_order() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5]);
+            let list = PyList::new(py, [2, 3, 5]);
             #[allow(clippy::reversed_empty_ranges)]
             list[2..1].extract::<Vec<i32>>().unwrap();
         })
@@ -677,7 +720,7 @@ mod tests {
     #[should_panic = "range start index 8 out of range for list of length 3"]
     fn test_list_index_trait_range_from_panic() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[2, 3, 5]);
+            let list = PyList::new(py, [2, 3, 5]);
             list[8..].extract::<Vec<i32>>().unwrap();
         })
     }
@@ -685,7 +728,7 @@ mod tests {
     #[test]
     fn test_list_del_item() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[1, 1, 2, 3, 5, 8]);
+            let list = PyList::new(py, [1, 1, 2, 3, 5, 8]);
             assert!(list.del_item(10).is_err());
             assert_eq!(1, list[0].extract::<i32>().unwrap());
             assert!(list.del_item(0).is_ok());
@@ -707,8 +750,8 @@ mod tests {
     #[test]
     fn test_list_set_slice() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[1, 1, 2, 3, 5, 8]);
-            let ins = PyList::new(py, &[7, 4]);
+            let list = PyList::new(py, [1, 1, 2, 3, 5, 8]);
+            let ins = PyList::new(py, [7, 4]);
             list.set_slice(1, 4, ins).unwrap();
             assert_eq!([1, 7, 4, 5, 8], list.extract::<[i32; 5]>().unwrap());
             list.set_slice(3, 100, PyList::empty(py)).unwrap();
@@ -719,7 +762,7 @@ mod tests {
     #[test]
     fn test_list_del_slice() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[1, 1, 2, 3, 5, 8]);
+            let list = PyList::new(py, [1, 1, 2, 3, 5, 8]);
             list.del_slice(1, 4).unwrap();
             assert_eq!([1, 5, 8], list.extract::<[i32; 3]>().unwrap());
             list.del_slice(1, 100).unwrap();
@@ -730,7 +773,7 @@ mod tests {
     #[test]
     fn test_list_contains() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[1, 1, 2, 3, 5, 8]);
+            let list = PyList::new(py, [1, 1, 2, 3, 5, 8]);
             assert_eq!(6, list.len());
 
             let bad_needle = 7i32.to_object(py);
@@ -747,7 +790,7 @@ mod tests {
     #[test]
     fn test_list_index() {
         Python::with_gil(|py| {
-            let list = PyList::new(py, &[1, 1, 2, 3, 5, 8]);
+            let list = PyList::new(py, [1, 1, 2, 3, 5, 8]);
             assert_eq!(0, list.index(1i32).unwrap());
             assert_eq!(2, list.index(2i32).unwrap());
             assert_eq!(3, list.index(3i32).unwrap());
