@@ -1,4 +1,3 @@
-// Copyright (c) 2017-present PyO3 Project and Contributors
 use crate::err::{self, PyDowncastError, PyErr, PyResult};
 use crate::exceptions::PyTypeError;
 #[cfg(feature = "experimental-inspect")]
@@ -7,9 +6,9 @@ use crate::internal_tricks::get_ssize_index;
 use crate::sync::GILOnceCell;
 use crate::type_object::PyTypeInfo;
 use crate::types::{PyAny, PyList, PyString, PyTuple, PyType};
-use crate::{ffi, PyNativeType, ToPyObject};
-use crate::{AsPyPointer, IntoPyPointer, Py, Python};
+use crate::{ffi, PyNativeType, PyObject, ToPyObject};
 use crate::{FromPyObject, PyTryFrom};
+use crate::{Py, Python};
 
 /// Represents a reference to a Python object supporting the sequence protocol.
 #[repr(transparent)]
@@ -24,11 +23,8 @@ impl PySequence {
     #[inline]
     pub fn len(&self) -> PyResult<usize> {
         let v = unsafe { ffi::PySequence_Size(self.as_ptr()) };
-        if v == -1 {
-            Err(PyErr::fetch(self.py()))
-        } else {
-            Ok(v as usize)
-        }
+        crate::err::error_on_minusone(self.py(), v)?;
+        Ok(v as usize)
     }
 
     /// Returns whether the sequence is empty.
@@ -129,17 +125,13 @@ impl PySequence {
     where
         I: ToPyObject,
     {
-        let py = self.py();
-        unsafe {
-            err::error_on_minusone(
-                py,
-                ffi::PySequence_SetItem(
-                    self.as_ptr(),
-                    get_ssize_index(i),
-                    item.to_object(py).as_ptr(),
-                ),
-            )
+        fn inner(seq: &PySequence, i: usize, item: PyObject) -> PyResult<()> {
+            err::error_on_minusone(seq.py(), unsafe {
+                ffi::PySequence_SetItem(seq.as_ptr(), get_ssize_index(i), item.as_ptr())
+            })
         }
+
+        inner(self, i, item.to_object(self.py()))
     }
 
     /// Deletes the `i`th element of self.
@@ -147,12 +139,9 @@ impl PySequence {
     /// This is equivalent to the Python statement `del self[i]`.
     #[inline]
     pub fn del_item(&self, i: usize) -> PyResult<()> {
-        unsafe {
-            err::error_on_minusone(
-                self.py(),
-                ffi::PySequence_DelItem(self.as_ptr(), get_ssize_index(i)),
-            )
-        }
+        err::error_on_minusone(self.py(), unsafe {
+            ffi::PySequence_DelItem(self.as_ptr(), get_ssize_index(i))
+        })
     }
 
     /// Assigns the sequence `v` to the slice of `self` from `i1` to `i2`.
@@ -160,17 +149,14 @@ impl PySequence {
     /// This is equivalent to the Python statement `self[i1:i2] = v`.
     #[inline]
     pub fn set_slice(&self, i1: usize, i2: usize, v: &PyAny) -> PyResult<()> {
-        unsafe {
-            err::error_on_minusone(
-                self.py(),
-                ffi::PySequence_SetSlice(
-                    self.as_ptr(),
-                    get_ssize_index(i1),
-                    get_ssize_index(i2),
-                    v.as_ptr(),
-                ),
+        err::error_on_minusone(self.py(), unsafe {
+            ffi::PySequence_SetSlice(
+                self.as_ptr(),
+                get_ssize_index(i1),
+                get_ssize_index(i2),
+                v.as_ptr(),
             )
-        }
+        })
     }
 
     /// Deletes the slice from `i1` to `i2` from `self`.
@@ -178,12 +164,9 @@ impl PySequence {
     /// This is equivalent to the Python statement `del self[i1:i2]`.
     #[inline]
     pub fn del_slice(&self, i1: usize, i2: usize) -> PyResult<()> {
-        unsafe {
-            err::error_on_minusone(
-                self.py(),
-                ffi::PySequence_DelSlice(self.as_ptr(), get_ssize_index(i1), get_ssize_index(i2)),
-            )
-        }
+        err::error_on_minusone(self.py(), unsafe {
+            ffi::PySequence_DelSlice(self.as_ptr(), get_ssize_index(i1), get_ssize_index(i2))
+        })
     }
 
     /// Returns the number of occurrences of `value` in self, that is, return the
@@ -194,13 +177,13 @@ impl PySequence {
     where
         V: ToPyObject,
     {
-        let r =
-            unsafe { ffi::PySequence_Count(self.as_ptr(), value.to_object(self.py()).as_ptr()) };
-        if r == -1 {
-            Err(PyErr::fetch(self.py()))
-        } else {
+        fn inner(seq: &PySequence, value: PyObject) -> PyResult<usize> {
+            let r = unsafe { ffi::PySequence_Count(seq.as_ptr(), value.as_ptr()) };
+            crate::err::error_on_minusone(seq.py(), r)?;
             Ok(r as usize)
         }
+
+        inner(self, value.to_object(self.py()))
     }
 
     /// Determines if self contains `value`.
@@ -211,13 +194,16 @@ impl PySequence {
     where
         V: ToPyObject,
     {
-        let r =
-            unsafe { ffi::PySequence_Contains(self.as_ptr(), value.to_object(self.py()).as_ptr()) };
-        match r {
-            0 => Ok(false),
-            1 => Ok(true),
-            _ => Err(PyErr::fetch(self.py())),
+        fn inner(seq: &PySequence, value: PyObject) -> PyResult<bool> {
+            let r = unsafe { ffi::PySequence_Contains(seq.as_ptr(), value.as_ptr()) };
+            match r {
+                0 => Ok(false),
+                1 => Ok(true),
+                _ => Err(PyErr::fetch(seq.py())),
+            }
         }
+
+        inner(self, value.to_object(self.py()))
     }
 
     /// Returns the first index `i` for which `self[i] == value`.
@@ -228,13 +214,13 @@ impl PySequence {
     where
         V: ToPyObject,
     {
-        let r =
-            unsafe { ffi::PySequence_Index(self.as_ptr(), value.to_object(self.py()).as_ptr()) };
-        if r == -1 {
-            Err(PyErr::fetch(self.py()))
-        } else {
+        fn inner(seq: &PySequence, value: PyObject) -> PyResult<usize> {
+            let r = unsafe { ffi::PySequence_Index(seq.as_ptr(), value.as_ptr()) };
+            crate::err::error_on_minusone(seq.py(), r)?;
             Ok(r as usize)
         }
+
+        inner(self, value.to_object(self.py()))
     }
 
     /// Returns a fresh list based on the Sequence.
@@ -402,7 +388,7 @@ impl Py<PySequence> {
 #[cfg(test)]
 mod tests {
     use crate::types::{PyList, PySequence, PyTuple};
-    use crate::{AsPyPointer, Py, PyObject, Python, ToPyObject};
+    use crate::{Py, PyObject, Python, ToPyObject};
 
     fn get_object() -> PyObject {
         // Convenience function for getting a single unique object
@@ -787,7 +773,7 @@ mod tests {
             let seq = ob.downcast::<PySequence>(py).unwrap();
             let repeat_seq = seq.repeat(3).unwrap();
             assert_eq!(6, repeat_seq.len().unwrap());
-            let repeated = vec!["foo", "bar", "foo", "bar", "foo", "bar"];
+            let repeated = ["foo", "bar", "foo", "bar", "foo", "bar"];
             for (el, rpt) in repeat_seq.iter().unwrap().zip(repeated.iter()) {
                 assert_eq!(*rpt, el.unwrap().extract::<String>().unwrap());
             }
@@ -833,7 +819,7 @@ mod tests {
             assert!(seq
                 .to_list()
                 .unwrap()
-                .eq(PyList::new(py, &["f", "o", "o"]))
+                .eq(PyList::new(py, ["f", "o", "o"]))
                 .unwrap());
             #[allow(deprecated)]
             {
@@ -851,7 +837,7 @@ mod tests {
             assert!(seq
                 .to_tuple()
                 .unwrap()
-                .eq(PyTuple::new(py, &["foo", "bar"]))
+                .eq(PyTuple::new(py, ["foo", "bar"]))
                 .unwrap());
             #[allow(deprecated)]
             {
