@@ -5,6 +5,7 @@ use pyo3::py_run;
 
 use pyo3::types::IntoPyDict;
 
+#[path = "../src/tests/common.rs"]
 mod common;
 
 #[pyclass(subclass)]
@@ -26,7 +27,7 @@ fn subclass() {
             None,
             Some(d),
         )
-        .map_err(|e| e.print(py))
+        .map_err(|e| e.display(py))
         .unwrap();
     });
 }
